<?php

namespace Barn2\Plugin\WC_Variation_Prices\Integration;

use Barn2\WVP_Lib\Registerable,
	Barn2\WVP_Lib\Service;

use Iconic_WAS_Fees;

defined( 'ABSPATH' ) || exit;

/**
 * Handles the integration with the WooCommerce Attribute Swatches by Iconic
 *
 * @package   Barn2\woocommerce-variation-prices
 * @author    Barn2 Plugins <support@barn2.com>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */
class Iconic_WAS implements Registerable, Service {

	/**
	 * Register the functionalities of the class
	 *
	 * @since 1.0.8
	 */
	public function register() {
		if ( class_exists( 'Iconic_WAS_Fees' ) ) {
			add_filter( 'wc_variation_prices_price_range', [ $this, 'get_price_range' ], 10, 2 );
			add_filter( 'wc_variation_prices_price_list', [ $this, 'get_price_list' ], 10, 2 );
		}
	}

	/**
	 * Get the attribute fees stored by WooCommerce Attribute Swatches by Iconic
	 *
	 * @param  WC_Product $product
	 * @return array
	 */
	public function get_fees( $product ) {
		$attributes = $product->get_attributes();

		$fees = [];

		foreach ( $attributes as $key => $attribute ) {
			$fees[ $key ] = Iconic_WAS_Fees::get_attribute_data( $attribute, $product->get_id() );
		}

		return $fees;
	}

	/**
	 * Filter the price range calculated by the Price_Range handler
	 * taking into account the attribute fees added by WooCommerce Attribute Swatches by Iconic
	 *
	 * @since 1.0.8
	 * @param  array[float] $range An associative array with `min` and `max`
	 * @param  WC_Product $product
	 * @return array[float] The
	 */
	public function get_price_range( $range, $product ) {
		$fees            = $this->get_fees( $product );
		$increment_range = array_reduce(
			$fees,
			function( $r, $a ) {
				$partial = array_reduce(
					$a['values'],
					function( $m, $v ) {
						return [ min( $m[0], $v['value'] ), max( $m[1], $v['value'] ) ];
					},
					[ INF, 0 ]
				);

				return [ $r[0] + $partial[0], $r[1] + $partial[1] ];
			},
			[ 0, 0 ]
		);

		return [
			'min' => (float) $range[ 'min' ] + $increment_range[0],
			'max' => (float) $range[ 'max' ] + $increment_range[1]
		];
	}

	public function get_price_list( $prices, $product ) {
		if ( is_a( $product, 'WC_Product_Grouped' ) ) {
			return $prices;
		}

		$fees = $this->get_fees( $product );

		foreach ( $prices['price'] as $variation_id => $price ) {
			$variation  = wc_get_product( $variation_id );
			$attributes = $variation->get_variation_attributes();
			$attributes = array_combine(
				array_map(
					function( $k ) {
						return str_replace( 'attribute_', '', $k );
					},
					array_keys( $attributes )
				),
				$attributes
			);

			$increment_range = array_reduce(
				$fees,
				function( $r, $a ) use( $attributes ) {
					$attribute = $attributes[ $a['slug'] ];
					if ( '' !== $attribute ) {
						$values  = $a['values'];
						$value   = $values[ $attribute ];
						$value   = $value['value'];
						$partial = [ $value, $value ];
					} else {
						$partial = array_reduce(
							$a['values'],
							function( $m, $v ) {
								return [ min( $m[0], $v['value'] ), max( $m[1], $v['value'] ) ];
							},
							[ INF, 0 ]
						);
					}

					return [ $r[0] + $partial[0], $r[1] + $partial[1] ];
				},
				[ 0, 0 ]
			);

			if ( $increment_range[0] !== $increment_range[1] ) {
				$prices['price'][ $variation_id ] = [
					'min' => $prices['price'][ $variation_id ] + $increment_range[0],
					'max' => $prices['price'][ $variation_id ] + $increment_range[1],
				];
			}
		}

		return $prices;
	}
}
